<?php
/**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace Magenest\Salesforce\Block\Adminhtml\OrderEdit\Tab;

use Magenest\Salesforce\Model\Connector;
use Magenest\Salesforce\Model\ResourceModel\Report\CollectionFactory as ReportCollectionFactory;
use Magenest\Salesforce\Model\Sync\Order;

/**
 * Class View
 * @package Magenest\Salesforce\Block\Adminhtml\OrderEdit\Tab
 */
class View extends \Magento\Backend\Block\Template implements \Magento\Backend\Block\Widget\Tab\TabInterface
{
    /**
     * Template
     *
     * @var string
     */
    protected $_template = 'tab/view/salesforce_order_info.phtml';

    /**
     * Core registry
     *
     * @var \Magento\Framework\Registry
     */
    protected $_coreRegistry = null;

    /** @var null  */
    protected $_orderIncrementId = null;

    /** @var null  */
    protected $_orderId = null;

    protected $_salesforceInstanceUrl = null;

    /** @var \Magenest\Salesforce\Model\ReportFactory  */
    protected $logFactory;

    /** @var ReportCollectionFactory  */
    protected $_reportCollectionFactory;

    /**
     * View constructor.
     * @param \Magento\Backend\Block\Template\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magenest\Salesforce\Model\ReportFactory $logFactory
     * @param ReportCollectionFactory $reportCollectionFactory
     * @param array $data
     */
    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Framework\Registry $registry,
        \Magenest\Salesforce\Model\ReportFactory $logFactory,
        ReportCollectionFactory $reportCollectionFactory,
        array $data = []
    ) {
        $this->logFactory = $logFactory;
        $this->_reportCollectionFactory = $reportCollectionFactory;
        $this->_coreRegistry = $registry;
        parent::__construct($context, $data);
    }

    /**
     * Retrieve order model instance
     *
     * @return \Magento\Sales\Model\Order
     */
    public function getOrder()
    {
        return $this->_coreRegistry->registry('current_order');
    }

    /**
     * Retrieve order increment id
     *
     * @return string
     */
    public function getOrderIncrementId()
    {
        if ($this->_orderIncrementId == null) {
            $this->_orderIncrementId = $this->getOrder()->getIncrementId();
        }
        return $this->_orderIncrementId;
    }

    /**
     * @return mixed
     */
    public function getOrderId()
    {
        if ($this->_orderId == null) {
            $this->_orderId = $this->getOrder()->getId();
        }
        return $this->_orderId;
    }
    /**
     * {@inheritdoc}
     */
    public function getTabLabel()
    {
        return __('Salesforce Integration');
    }

    /**
     * {@inheritdoc}
     */
    public function getTabTitle()
    {
        return __('Sync History');
    }

    /**
     * {@inheritdoc}
     */
    public function canShowTab()
    {
        return true;
    }

    /**
     * {@inheritdoc}
     */
    public function isHidden()
    {
        return false;
    }

    /**
     * @return string
     */
    public function getCreatedAt()
    {
        $log = $this->_reportCollectionFactory->create()
            ->addFieldToFilter('salesforce_table', ['IN' => ['Order', 'Opportunity']])
            ->addFieldToFilter('magento_id', $this->getOrderIncrementId())
            ->getFirstItem();
        return $log->getData('datetime') ? $this->formatDate($log->getData('datetime'), \IntlDateFormatter::MEDIUM, true) : 'Never';
    }

    /**
     * Get customer creation date
     *
     * @return string
     */
    public function getLastUpdatedAt()
    {
        $log = $this->_reportCollectionFactory->create()
            ->addFieldToFilter('salesforce_table', ['IN' => ['Order', 'Opportunity']])
            ->addFieldToFilter('magento_id', $this->getOrderIncrementId())
            ->getLastItem();
        return $log->getData('datetime') ? $this->formatDate($log->getData('datetime'), \IntlDateFormatter::MEDIUM, true) : 'Never';
    }

    /**
     * @return string
     */
    public function getSalesforceId()
    {
        $orderData = $this->getOrder()->getData();
        if (!empty($orderData) && isset($orderData[Order::SALESFORCE_ORDER_ATTRIBUTE_CODE])) {
            return $orderData[Order::SALESFORCE_ORDER_ATTRIBUTE_CODE];
        }
        return '';
    }

    /**
     * @return string
     */
    public function getSalesforceOpportunityId()
    {
        $reportModel = $this->getSalesforceInfo($this->getOrderIncrementId(), ['Opportunity']);
        if ($reportModel->getRecordId()) {
            return $reportModel->getRecordId();
        }
        return '';
    }

    /**
     * @param string[] $types
     * @param $magentoId
     * @return \Magento\Framework\DataObject
     */
    public function getSalesforceInfo($magentoId, $types = ['Order'])
    {
        return $this->_reportCollectionFactory->create()
            ->addFieldToFilter('salesforce_table', ['IN' => $types])
            ->addFieldToFilter('magento_id', $magentoId)
            ->getLastItem();
    }

    /**
     * @return mixed
     */
    private function getSalesforceInstanceUrl()
    {
        if ($this->_salesforceInstanceUrl == null) {
            $this->_salesforceInstanceUrl = $this->_scopeConfig->getValue(Connector::XML_PATH_SALESFORCE_INSTANCE_URL);
        }
        return $this->_salesforceInstanceUrl;
    }

    /**
     * @return string
     */
    public function getSalesforceUrl()
    {
        return $this->getSalesforceInstanceUrl() . '/' . $this->getSalesforceId();
    }

    /**
     * @return string
     */
    public function getSalesforceOpportunityUrl()
    {
        return $this->getSalesforceInstanceUrl() . '/' . $this->getSalesforceOpportunityId();
    }

    /**
     * @return \Magento\Framework\Model\ResourceModel\Db\Collection\AbstractCollection
     */
    public function getSyncLog()
    {
        return $this->_reportCollectionFactory->create()
            ->addFieldToFilter('salesforce_table', ['IN' => ['Order', 'Opportunity']])
            ->addFieldToFilter('magento_id', $this->getOrderIncrementId())
            ->addOrder('datetime', 'DESC')
            ->setPageSize(10)
            ->setCurPage(1);
    }

    /**
     * @return string
     */
    public function getSyncButtonUrl()
    {
        return $this->getUrl(
            'salesforce/sync/order',
            [
                'increment_id' => $this->getOrderIncrementId(),
                'order_id' => $this->getOrderId()
            ]
        );
    }

    /**
     * @return string
     */
    public function getSyncOpportunityButtonUrl()
    {
        return $this->getUrl(
            'salesforce/sync/opportunity',
            [
                'increment_id' => $this->getOrderIncrementId(),
                'order_id' => $this->getOrderId()
            ]
        );
    }
}
